// [License]
// The Ariba-Underlay Copyright
//
// Copyright (c) 2008-2009, Institute of Telematics, Universität Karlsruhe (TH)
//
// Institute of Telematics
// Universität Karlsruhe (TH)
// Zirkel 2, 76128 Karlsruhe
// Germany
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE INSTITUTE OF TELEMATICS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ARIBA PROJECT OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation
// are those of the authors and should not be interpreted as representing
// official policies, either expressed or implied, of the Institute of
// Telematics.
// [License]

#ifndef ENDPOINTDESCRIPTOR_H_
#define ENDPOINTDESCRIPTOR_H_

#include "ariba/utility/types.h"
#include "ariba/utility/serialization.h"
#include "ariba/communication/modules/network/ip/IPv4Locator.h"
#include <string>
#include <set>

using std::string;
using std::set;
using ariba::utility::Locator;
using ariba::communication::IPv4Locator;

namespace ariba {
namespace communication {

using_serialization;

class EndpointDescriptor : public VSerializeable {
	VSERIALIZEABLE;

	friend class BaseCommunication;

public:
	/**
	 * The default constructor.
	 */
	EndpointDescriptor();

	EndpointDescriptor(const EndpointDescriptor& rh);
	EndpointDescriptor(const Locator* _locator);

	/**
	 * The destructor.
	 */
	virtual ~EndpointDescriptor();

	/**
	 * An unspecified end-point
	 */
	static const EndpointDescriptor UNSPECIFIED;

	/**
	 * Returns true, if the descriptor is unspecified.
	 *
	 * @return True, if the descriptor is unspecified.
	 */
	bool isUnspecified() const;

	virtual string toString() const;
	static EndpointDescriptor* fromString( string str );

	bool operator==( const EndpointDescriptor& rh ) const;

private:
	bool isUnspec;
	IPv4Locator* locator;
};

}} // namespace ariba, communication

sznBeginDefault( ariba::communication::EndpointDescriptor, X ) {
	uint16_t unspec = isUnspec; //TODO: was uint8_t

	X && unspec && VO(locator);

	// when deserializing reset unspec flag
	if (X.isDeserializer()) isUnspec = unspec;
} sznEnd();

#endif /*ENDPOINTDESCRIPTOR_H_*/
